extern crate clap;
use clap::{App, Arg};

use std::io::prelude::*;
use std::net::{TcpListener, TcpStream};

use echo_ip::ThreadPool;

fn main() {
    let matches = App::new("EchoIP")
        .version("1.0")
        .author("Yaser Hsueh <xueye404@icloud.com>")
        .about("https://www.simaek.com")
        .arg(
            Arg::with_name("port")
                .short("p")
                .long("port")
                .value_name("port")
                .help("listening port")
                .takes_value(true),
        )
        .arg(
            Arg::with_name("server")
                .short("s")
                .long("server")
                .value_name("host")
                .help("listening address")
                .takes_value(true),
        )
        .get_matches();

    let host = matches.value_of("server").unwrap_or("0.0.0.0");
    let port = matches.value_of("port").unwrap_or("8888");
    let bind_address = host.to_owned() + ":" + port;

    let listener = TcpListener::bind(&bind_address);
    let pool = ThreadPool::new(4);

    match listener {
        Ok(n) => {
            println!("-----------------------------------------------------------");
            println!("Server listening on {}", &port);
            println!("-----------------------------------------------------------");
            for stream in n.incoming() {
                let stream = stream.unwrap();
                //thread::spawn(|| {
                pool.execute(|| {
                    handle_connection(stream);
                });
            }
        }
        Err(_e) => println!("Address already in use: {}", bind_address),
    }
}

fn handle_connection(mut stream: TcpStream) {
    let mut buffer = [0; 1024];
    let remote_addr = stream.peer_addr().unwrap().ip();
    println!("{}", stream.read(&mut buffer).unwrap());
    println!("{}", String::from_utf8_lossy(&buffer[..]));
    //stream.read(&mut buffer).unwrap();
    let get = b"GET / HTTP/1.1\r\n";
    let (status_line, contents) = if buffer.starts_with(get) {
        ("HTTP/1.1 200 OK", remote_addr.to_string())
    } else {
        ("HTTP/1.1 404 NOT FOUND", "URL ERROR".to_string())
    };
    println!("echo {}", remote_addr);
    let response = format!("{}\r\n\r\n{}\r\n", status_line, contents);
    stream.write(response.as_bytes()).unwrap();
    stream.flush().unwrap();
}
